@@ -1518,9 +1518,11 @@ void forestryTree::printTree(){
 
 void forestryTree::getOOBindex(
     std::vector<size_t> &outputOOBIndex,
-    size_t nRows
+    std::vector<size_t> &allIndex
 ){
 
+  size_t nRows = allIndex.size();
+
   // Generate union of splitting and averaging dataset
   std::sort(
     getSplittingIndex()->begin(),
@@ -1530,6 +1532,10 @@ void forestryTree::getOOBindex(
     getAveragingIndex()->begin(),
     getAveragingIndex()->end()
   );
+  std::sort(
+          allIndex.begin(),
+          allIndex.end()
+  );
 
   std::vector<size_t> allSampledIndex(
       getSplittingIndex()->size() + getAveragingIndex()->size()
@@ -1545,16 +1551,6 @@ void forestryTree::getOOBindex(
 
   allSampledIndex.resize((unsigned long) (it - allSampledIndex.begin()));
 
-  // Generate a vector of all index based on nRows
-  struct IncGenerator {
-    size_t current_;
-    IncGenerator(size_t start): current_(start) {}
-    size_t operator()() { return current_++; }
-  };
-  std::vector<size_t> allIndex(nRows);
-  IncGenerator g(0);
-  std::generate(allIndex.begin(), allIndex.end(), g);
-
   // OOB index is the set difference between sampled index and all index
   std::vector<size_t> OOBIndex(nRows);
 
@@ -1579,9 +1575,10 @@ void forestryTree::getOOBindex(
 
 void forestryTree::getDoubleOOBIndex(
     std::vector<size_t> &outputOOBIndex,
-    size_t nRows
+    std::vector<size_t> &allIndex
 ){
 
+  size_t nRows = allIndex.size();
   // This function is intended to be used with OOB honesty
   // using a double bootstrap in order to get the observations which
   // were not selected in the first bootstrap (the splitting set),
@@ -1600,6 +1597,11 @@ void forestryTree::getDoubleOOBIndex(
     getAveragingIndex()->end()
   );
 
+  std::sort(
+          allIndex.begin(),
+          allIndex.end()
+  );
+
   std::vector<size_t> allSampledIndex(
       getSplittingIndex()->size() + getAveragingIndex()->size()
   );
@@ -1614,16 +1616,6 @@ void forestryTree::getDoubleOOBIndex(
 
   allSampledIndex.resize((unsigned long) (it - allSampledIndex.begin()));
 
-  // Generate a vector of all index based on nRows
-  struct IncGenerator {
-    size_t current_;
-    IncGenerator(size_t start): current_(start) {}
-    size_t operator()() { return current_++; }
-  };
-  std::vector<size_t> allIndex(nRows);
-  IncGenerator g(0);
-  std::generate(allIndex.begin(), allIndex.end(), g);
-
   // OOB index is the set difference between sampled index and all index
   std::vector<size_t> OOBIndex(nRows);
 
@@ -1647,23 +1639,23 @@ void forestryTree::getDoubleOOBIndex(
 
 void forestryTree::getOOBhonestIndex(
     std::vector<size_t> &outputOOBIndex,
-    size_t nRows
+    std::vector<size_t> &allIndex
 ){
 
+  size_t nRows = allIndex.size();
+
   // Generate union of splitting and averaging dataset
   std::sort(
     getAveragingIndex()->begin(),
     getAveragingIndex()->end()
   );
 
-  struct IncGenerator {
-    size_t current_;
-    IncGenerator(size_t start): current_(start) {}
-    size_t operator()() { return current_++; }
-  };
-  std::vector<size_t> allIndex(nRows);
-  IncGenerator g(0);
-  std::generate(allIndex.begin(), allIndex.end(), g);
+  // Super annoying problem, have to sort before doing set_diff
+  std::sort(
+          allIndex.begin(),
+          allIndex.end()
+  );
+
 
   // OOB index is the set difference between sampled index and all index
   std::vector<size_t> OOBIndex(nRows);
@@ -1677,6 +1669,8 @@ void forestryTree::getOOBhonestIndex(
   );
   OOBIndex.resize((unsigned long) (it - OOBIndex.begin()));
 
+  //std::cout << "OOB index inside get OOB HonestIndex" << std::endl;
+  //  print_vector(OOBIndex);
   for (
       std::vector<size_t>::iterator it_ = OOBIndex.begin();
       it_ != OOBIndex.end();
@@ -1689,8 +1683,9 @@ void forestryTree::getOOBhonestIndex(
 void forestryTree::getOOGIndex(
     std::vector<size_t> &outputOOBIndex,
     std::vector<size_t> groupMemberships,
-    size_t nRows
-) {
+    std::vector<size_t> &allIndex
+){
+
   // For a given tree, we cycle through all averaging indices and get their
   // group memberships. Then we take the set of observations which are in groups
   // which haven't been seen by the current tree, and output this to outputOOBIndex
@@ -1699,14 +1694,10 @@ void forestryTree::getOOGIndex(
     getAveragingIndex()->end()
   );
 
-  struct IncGenerator {
-    size_t current_;
-    IncGenerator(size_t start): current_(start) {}
-    size_t operator()() { return current_++; }
-  };
-  std::vector<size_t> allIndex(nRows);
-  IncGenerator g(0);
-  std::generate(allIndex.begin(), allIndex.end(), g);
+  std::sort(
+          allIndex.begin(),
+          allIndex.end()
+  );
 
   // Add all in sample groups to a set
   std::set<size_t> in_sample_groups;
@@ -1737,31 +1728,46 @@ void forestryTree::getOOBPrediction(
     bool doubleOOB,
     size_t nodesizeStrictAvg,
     std::vector< std::vector<double> >* xNew,
-    arma::Mat<double>* weightMatrix
+    arma::Mat<double>* weightMatrix,
+    const std::vector<size_t>& training_idx
 ){
 
   std::vector<size_t> OOBIndex;
-
+  bool use_training_idx = !(training_idx.size() == 0);
   // OOB with and without using the OOB set as the averaging set requires
   // different sets of trees to be used (we want the set of trees)
   // without the averaging set
+
+  std::vector<size_t> allIndex(trainingData->getNumRows());
+  if (!use_training_idx) {
+      std::iota(allIndex.begin(), allIndex.end(), 0);
+  } else {
+      allIndex = training_idx;
+      //std::cout << "All Index (size) "<< allIndex.size()  << std::endl;
+      //print_vector(allIndex);
+  }
+    //std::cout << "tree avging indices" << std::endl;
+    //print_vector(*getAveragingIndex());
+
   if (trainingData->getGroups()->at(0) != 0) {
     std::vector<size_t> group_membership_vector = *(trainingData->getGroups());
-    getOOGIndex(OOBIndex, group_membership_vector, trainingData->getNumRows());
+    getOOGIndex(OOBIndex, group_membership_vector, allIndex);
   } else {
     if (OOBhonest) {
       if (doubleOOB) {
         // Get setDiff(1:nrow(x), union(splittingIndices, averagingIndices))
-        getDoubleOOBIndex(OOBIndex, trainingData->getNumRows());
+        getDoubleOOBIndex(OOBIndex, allIndex);
       } else {
         // Get setDiff(1:nrow(x), averagingIndices)
-        getOOBhonestIndex(OOBIndex, trainingData->getNumRows());
+        getOOBhonestIndex(OOBIndex, allIndex);
       }
     } else {
       // Standard OOB indices in the non honest case
-      getOOBindex(OOBIndex, trainingData->getNumRows());
+      getOOBindex(OOBIndex, allIndex);
     }
   }
+  //std::cout << "OOB Index" << std::endl;
+  //print_vector(OOBIndex);
 
   // Xnew has first access being the feature selection and second access being
   // the observation selection.
@@ -1779,20 +1785,49 @@ void forestryTree::getOOBPrediction(
   std::vector<double> currentTreePrediction(OOBIndex.size());
   std::vector<int>* currentTreeTerminalNodes = nullptr;
   std::vector< std::vector<double> > currentTreeCoefficients(OOBIndex.size());
-
   std::vector< std::vector<double> > xnew(trainingData->getNumColumns());
 
+  // If using training indices, need to make sure we get the correct rows in
+  // the new data, so make a map from OOB Indices for the tree -> row id in xnew
+  std::vector<size_t> idx_to_use;
+  std::map<size_t, size_t> map_trainidx;
+  std::vector<size_t> indexInTrain(OOBIndex.size());
+
+  if (use_training_idx) {
+
+      for (size_t i = 0; i < training_idx.size(); i++) {
+          map_trainidx[training_idx[i]] = i;
+      }
+
+      // Print the map
+      //for (const auto& [key, value] : map_trainidx) {
+      //    std::cout << '[' << key << "] = " << value << "; ";
+      //}
+      //std::cout << std::endl << "Idx to use" << std::endl;
+      //print_vector(OOBIndex);
+
+      for (size_t k = 0; k < OOBIndex.size(); k++) {
+          indexInTrain[k] = ((size_t) map_trainidx.at(OOBIndex[k]));
+      }
+  }
+  //std::cout << std::endl << "Idx to use before doing iterator" << std::endl;
+  //print_vector(indexInTrain);
+
   for (size_t k = 0; k < trainingData->getNumColumns(); k++)
     {
       // Populate all values of the Kth feature
-      for (
-          std::vector<size_t>::iterator it=OOBIndex.begin();
-          it!=OOBIndex.end();
-          ++it
-      ) {
-        xnew[k].push_back((*OOBSampleObservations_)[k][*it]);
+      if (use_training_idx) {
+          for ( const auto& row_idx : OOBIndex) {
+              xnew[k].push_back((*OOBSampleObservations_)[k][map_trainidx.at(row_idx)]);
+          }
+      } else {
+          for (const auto& row_idx : OOBIndex) {
+              xnew[k].push_back((*OOBSampleObservations_)[k][row_idx]);
+          }
       }
     }
+    //std::cout << std::endl << "Idx to use after doing iterator" << std::endl;
+    //print_vector(indexInTrain);
 
   // Run predict on the new feature corresponding to all out of bag observations
   predict(
@@ -1805,28 +1840,35 @@ void forestryTree::getOOBPrediction(
     false,
     44,
     nodesizeStrictAvg,
-    &OOBIndex
+    use_training_idx ? &indexInTrain : &OOBIndex
   );
 
-
-  // arma::Mat<double> curWeightMatrix;
-  // size_t nrow = OOBIndex.size(); // number of features to be predicted
-  // size_t ncol = trainingData->getNumRows(); // number of train data
-  // curWeightMatrix.resize(nrow, ncol); // initialize the space for the matrix
-  // curWeightMatrix.zeros(nrow, ncol);// set it all to 0
-
-
-  for (size_t i = 0; i < OOBIndex.size(); i++) {
-    // Update the global OOB vector
-    outputOOBPrediction[OOBIndex[i]] += currentTreePrediction[i];
-    outputOOBCount[OOBIndex[i]] += 1;
-
-    // Check weight matrix before we updte
-    // if (weightMatrix) {
-    //   for (size_t j = 0; j < ncol; j++) {
-    //     (*weightMatrix)(OOBIndex[i], j) += curWeightMatrix(i, j);
-    //   }
-    // }
+    //std::cout << "Index in train" << std::endl;
+    //print_vector(indexInTrain);
+    //std::cout << "OOB Index" << std::endl;
+    //print_vector(OOBIndex);
+  // Now take only the OOB entries in the predictions
+  // If we are using training indices, use those
+  if (use_training_idx) {
+      //std::cout << "tree predictions before selection" << std::endl;
+      //print_vector(currentTreePrediction);
+      //std::cout << "globalOOB predictions before selection " << std::endl;
+      //print_vector(outputOOBPrediction);
+      //std::cout << "size of idx to use " << OOBIndex.size() << std::endl;
+      for (size_t i = 0; i < OOBIndex.size(); i++) {
+          // Update the global OOB vector
+          //std::cout << "putting" << currentTreePrediction[i] << " at idx " << map_trainidx.at(OOBIndex[i]) << " in global vec" << std::endl;
+          outputOOBPrediction[map_trainidx.at(OOBIndex[i])] += currentTreePrediction[i];
+          outputOOBCount[map_trainidx.at(OOBIndex[i])] += 1;
+      }
+      //std::cout << "globalOOB predictions after selection " << std::endl;
+      //print_vector(outputOOBPrediction);
+  } else {
+      for (size_t i = 0; i < OOBIndex.size(); i++) {
+          // Update the global OOB vector
+          outputOOBPrediction[OOBIndex[i]] += currentTreePrediction[i];
+          outputOOBCount[OOBIndex[i]] += 1;
+      }
   }
 }
 
@@ -1841,8 +1883,11 @@ void forestryTree::getShuffledOOBPrediction(
   // Gives OOB prediction with shuffleFeature premuted randomly
   // For use in determining variable importance
 
+  std::vector<size_t> allIndex(trainingData->getNumRows());
+  std::iota(allIndex.begin(), allIndex.end(), 0);
+
   std::vector<size_t> OOBIndex;
-  getOOBindex(OOBIndex, trainingData->getNumRows());
+  getOOBindex(OOBIndex, allIndex);
 
   std::vector<size_t> shuffledOOBIndex = OOBIndex;
   std::shuffle(shuffledOOBIndex.begin(), shuffledOOBIndex.end(), random_number_generator);
