@@ -33,6 +33,20 @@ void group_out_sample(
     std::mt19937_64& random_number_generator
 );
 
+size_t bin_to_idx(
+    std::vector<bool> binary
+);
+
+size_t idx_to_bin(
+    size_t idx,
+    size_t i
+);
+
+std::vector<bool> get_symmetric_feat_signs(
+    std::vector<double> feature_vector,
+    std::vector<size_t> symmmetric_indices
+);
+
 struct tree_info {
   std::vector< int > var_id;
   // contains the variable id for a splitting node and the negative number of
@@ -74,6 +88,11 @@ struct monotonic_info {
   double upper_bound;
   double lower_bound;
 
+  // Two more upper bounds, these are only used when we are doing symmetric splits
+  // and must enforce monotonicity on both the positive and negative outcomes
+  // within a single node.
+  double upper_bound_neg;
+  double lower_bound_neg;
   // This flag indicates whether or not to enforce monotonicity on the averaging
   // set as well as the splitting set
   bool monotoneAvg;
@@ -83,4 +102,26 @@ struct monotonic_info {
   };
 };
 
+// Contains the information for symmetric splitting options
+struct symmetric_info {
+
+  // Contains the (0 indexed) indices of the variables that we are enforcing
+  // symmetry on
+  std::vector<size_t> symmetric_variables;
+
+  std::vector<double> upper_bounds;
+  std::vector<double> lower_bounds;
+
+  // Contains the positive an negative pseudo outcomes for each combination of
+  // signs of the symmetric features.
+  // This is of size 2^|S| where S is the set of features with enforced symmetry
+  // The integer expansion of the feature signs gives the index of the corresponding
+  // pseudo outcome.
+  // i.e. for two features, 0 = 00 is both features negative, 1 = 10 is feature
+  // 1 positive feature 2 negative, 2 = 01 is feature 1 negative feature 2 positive,
+  // and 3 = 11 is feature 1 and 2 positive.
+  std::vector<double> pseudooutcomes;
+
+};
+
 #endif //FORESTRYCPP_UTILS_H
